# Warning: this file is auto-generated. Do not edit.

from ctypes import CFUNCTYPE, POINTER, Structure, c_char_p, c_float, c_int, c_long, c_longlong, c_uint32, c_void_p

import ctypes
from typing import Optional
from OpenGL import GLX

from ..array_field import *
from ..enums import *
from ..typedefs import *
from ..version import *


# Forward declaration of a Wayland structure
class wl_display(Structure):
    pass
    

class timespec(Structure):
    _fields_ = [
        ("tv_sec", c_longlong),  # TODO: is this the correct type?
        ("tv_nsec", c_long),
    ]


KHR_vulkan_swapchain_format_list = 1
KHR_vulkan_swapchain_format_list_SPEC_VERSION = 4
KHR_VULKAN_SWAPCHAIN_FORMAT_LIST_EXTENSION_NAME = "XR_KHR_vulkan_swapchain_format_list"
KHR_opengl_enable = 1
KHR_opengl_enable_SPEC_VERSION = 10
KHR_OPENGL_ENABLE_EXTENSION_NAME = "XR_KHR_opengl_enable"
KHR_vulkan_enable = 1
KHR_vulkan_enable_SPEC_VERSION = 8
KHR_VULKAN_ENABLE_EXTENSION_NAME = "XR_KHR_vulkan_enable"
KHR_convert_timespec_time = 1
KHR_convert_timespec_time_SPEC_VERSION = 1
KHR_CONVERT_TIMESPEC_TIME_EXTENSION_NAME = "XR_KHR_convert_timespec_time"
KHR_vulkan_enable2 = 1
KHR_vulkan_enable2_SPEC_VERSION = 2
KHR_VULKAN_ENABLE2_EXTENSION_NAME = "XR_KHR_vulkan_enable2"
FB_foveation_vulkan = 1
FB_foveation_vulkan_SPEC_VERSION = 1
FB_FOVEATION_VULKAN_EXTENSION_NAME = "XR_FB_foveation_vulkan"
FB_swapchain_update_state_vulkan = 1
FB_swapchain_update_state_vulkan_SPEC_VERSION = 1
FB_SWAPCHAIN_UPDATE_STATE_VULKAN_EXTENSION_NAME = "XR_FB_swapchain_update_state_vulkan"
META_vulkan_swapchain_create_info = 1
META_vulkan_swapchain_create_info_SPEC_VERSION = 1
META_VULKAN_SWAPCHAIN_CREATE_INFO_EXTENSION_NAME = "XR_META_vulkan_swapchain_create_info"


class VulkanSwapchainFormatListCreateInfoKHR(Structure):
    def __init__(
        self,
        view_format_count: Optional[int] = None,
        view_formats: ArrayFieldParamType[c_int] = None,
        next: c_void_p = None,
        type: StructureType = StructureType.VULKAN_SWAPCHAIN_FORMAT_LIST_CREATE_INFO_KHR,
    ) -> None:
        view_format_count, view_formats = array_field_helper(
            c_int, view_format_count, view_formats)
        super().__init__(
            view_format_count=view_format_count,
            _view_formats=view_formats,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.VulkanSwapchainFormatListCreateInfoKHR(view_format_count={repr(self.view_format_count)}, view_formats={repr(self._view_formats)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.VulkanSwapchainFormatListCreateInfoKHR(view_format_count={self.view_format_count}, view_formats={self._view_formats}, next={self.next}, type={self.type})"

    @property
    def view_formats(self):
        if self.view_format_count == 0:
            return (c_int * 0)()
        else:
            return (c_int * self.view_format_count).from_address(
                ctypes.addressof(self._view_formats.contents))

    @view_formats.setter
    def view_formats(self, value):
        self.view_format_count, self._view_formats = array_field_helper(
            c_int, None, value)

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("view_format_count", c_uint32),
        ("_view_formats", POINTER(c_int)),
    ]


class GraphicsBindingOpenGLXlibKHR(Structure):
    def __init__(
        self,
        x_display: POINTER(GLX.Display) = None,
        visualid: int = 0,
        glx_fbconfig: GLX.GLXFBConfig = None,
        glx_drawable: GLX.GLXDrawable = 0,
        glx_context: GLX.GLXContext = None,
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_BINDING_OPENGL_XLIB_KHR,
    ) -> None:
        super().__init__(
            x_display=x_display,
            visualid=visualid,
            glx_fbconfig=glx_fbconfig,
            glx_drawable=glx_drawable,
            glx_context=glx_context,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsBindingOpenGLXlibKHR(x_display={repr(self.x_display)}, visualid={repr(self.visualid)}, glx_fbconfig={repr(self.glx_fbconfig)}, glx_drawable={repr(self.glx_drawable)}, glx_context={repr(self.glx_context)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsBindingOpenGLXlibKHR(x_display={self.x_display}, visualid={self.visualid}, glx_fbconfig={self.glx_fbconfig}, glx_drawable={self.glx_drawable}, glx_context={self.glx_context}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("x_display", POINTER(GLX.Display)),
        ("visualid", c_uint32),
        ("glx_fbconfig", GLX.GLXFBConfig),
        ("glx_drawable", GLX.GLXDrawable),
        ("glx_context", GLX.GLXContext),
    ]


class GraphicsBindingOpenGLXcbKHR(Structure):
    def __init__(
        self,
        connection: POINTER(c_int) = None,
        screen_number: int = 0,
        fbconfigid: int = 0,
        visualid: int = 0,
        glx_drawable: int = 0,
        glx_context: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_BINDING_OPENGL_XCB_KHR,
    ) -> None:
        super().__init__(
            connection=connection,
            screen_number=screen_number,
            fbconfigid=fbconfigid,
            visualid=visualid,
            glx_drawable=glx_drawable,
            glx_context=glx_context,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsBindingOpenGLXcbKHR(connection={repr(self.connection)}, screen_number={repr(self.screen_number)}, fbconfigid={repr(self.fbconfigid)}, visualid={repr(self.visualid)}, glx_drawable={repr(self.glx_drawable)}, glx_context={repr(self.glx_context)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsBindingOpenGLXcbKHR(connection={self.connection}, screen_number={self.screen_number}, fbconfigid={self.fbconfigid}, visualid={self.visualid}, glx_drawable={self.glx_drawable}, glx_context={self.glx_context}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("connection", POINTER(c_int)),
        ("screen_number", c_uint32),
        ("fbconfigid", c_int),
        ("visualid", c_int),
        ("glx_drawable", c_int),
        ("glx_context", c_int),
    ]


class GraphicsBindingOpenGLWaylandKHR(Structure):
    def __init__(
        self,
        display: POINTER(wl_display) = None,
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_BINDING_OPENGL_WAYLAND_KHR,
    ) -> None:
        super().__init__(
            display=display,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsBindingOpenGLWaylandKHR(display={repr(self.display)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsBindingOpenGLWaylandKHR(display={self.display}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("display", POINTER(wl_display)),
    ]


class SwapchainImageOpenGLKHR(Structure):
    def __init__(
        self,
        image: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.SWAPCHAIN_IMAGE_OPENGL_KHR,
    ) -> None:
        super().__init__(
            image=image,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.SwapchainImageOpenGLKHR(image={repr(self.image)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.SwapchainImageOpenGLKHR(image={self.image}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("image", c_uint32),
    ]


class GraphicsRequirementsOpenGLKHR(Structure):
    def __init__(
        self,
        min_api_version_supported: Version = Version(),
        max_api_version_supported: Version = Version(),
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_REQUIREMENTS_OPENGL_KHR,
    ) -> None:
        super().__init__(
            min_api_version_supported=min_api_version_supported.number(),
            max_api_version_supported=max_api_version_supported.number(),
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsRequirementsOpenGLKHR(min_api_version_supported={repr(self.min_api_version_supported)}, max_api_version_supported={repr(self.max_api_version_supported)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsRequirementsOpenGLKHR(min_api_version_supported={self.min_api_version_supported}, max_api_version_supported={self.max_api_version_supported}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("min_api_version_supported", VersionNumber),
        ("max_api_version_supported", VersionNumber),
    ]


PFN_xrGetOpenGLGraphicsRequirementsKHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, POINTER(GraphicsRequirementsOpenGLKHR))


class GraphicsBindingVulkanKHR(Structure):
    def __init__(
        self,
        instance: int = 0,
        physical_device: int = 0,
        device: int = 0,
        queue_family_index: int = 0,
        queue_index: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_BINDING_VULKAN_KHR,
    ) -> None:
        super().__init__(
            instance=instance,
            physical_device=physical_device,
            device=device,
            queue_family_index=queue_family_index,
            queue_index=queue_index,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsBindingVulkanKHR(instance={repr(self.instance)}, physical_device={repr(self.physical_device)}, device={repr(self.device)}, queue_family_index={repr(self.queue_family_index)}, queue_index={repr(self.queue_index)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsBindingVulkanKHR(instance={self.instance}, physical_device={self.physical_device}, device={self.device}, queue_family_index={self.queue_family_index}, queue_index={self.queue_index}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("instance", c_int),
        ("physical_device", c_int),
        ("device", c_int),
        ("queue_family_index", c_uint32),
        ("queue_index", c_uint32),
    ]


class SwapchainImageVulkanKHR(Structure):
    def __init__(
        self,
        image: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.SWAPCHAIN_IMAGE_VULKAN_KHR,
    ) -> None:
        super().__init__(
            image=image,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.SwapchainImageVulkanKHR(image={repr(self.image)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.SwapchainImageVulkanKHR(image={self.image}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("image", c_int),
    ]


class GraphicsRequirementsVulkanKHR(Structure):
    def __init__(
        self,
        min_api_version_supported: Version = Version(),
        max_api_version_supported: Version = Version(),
        next: c_void_p = None,
        type: StructureType = StructureType.GRAPHICS_REQUIREMENTS_VULKAN_KHR,
    ) -> None:
        super().__init__(
            min_api_version_supported=min_api_version_supported.number(),
            max_api_version_supported=max_api_version_supported.number(),
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.GraphicsRequirementsVulkanKHR(min_api_version_supported={repr(self.min_api_version_supported)}, max_api_version_supported={repr(self.max_api_version_supported)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.GraphicsRequirementsVulkanKHR(min_api_version_supported={self.min_api_version_supported}, max_api_version_supported={self.max_api_version_supported}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("min_api_version_supported", VersionNumber),
        ("max_api_version_supported", VersionNumber),
    ]


PFN_xrGetVulkanInstanceExtensionsKHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, c_uint32, POINTER(c_uint32), c_char_p)

PFN_xrGetVulkanDeviceExtensionsKHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, c_uint32, POINTER(c_uint32), c_char_p)

PFN_xrGetVulkanGraphicsDeviceKHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, c_int, POINTER(c_int))

PFN_xrGetVulkanGraphicsRequirementsKHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, POINTER(GraphicsRequirementsVulkanKHR))

PFN_xrConvertTimespecTimeToTimeKHR = CFUNCTYPE(Result.ctype(), Instance, POINTER(timespec), POINTER(Time))

PFN_xrConvertTimeToTimespecTimeKHR = CFUNCTYPE(Result.ctype(), Instance, Time, POINTER(timespec))

VulkanInstanceCreateFlagsKHRCInt = Flags64

class VulkanInstanceCreateFlagsKHR(FlagBase):
    NONE = 0x00000000

VulkanDeviceCreateFlagsKHRCInt = Flags64

class VulkanDeviceCreateFlagsKHR(FlagBase):
    NONE = 0x00000000


class VulkanInstanceCreateInfoKHR(Structure):
    def __init__(
        self,
        system_id: SystemId = 0,
        create_flags: VulkanInstanceCreateFlagsKHR = VulkanInstanceCreateFlagsKHR(),
        pfn_get_instance_proc_addr: int = 0,
        vulkan_create_info: POINTER(c_int) = None,
        vulkan_allocator: POINTER(c_int) = None,
        next: c_void_p = None,
        type: StructureType = StructureType.VULKAN_INSTANCE_CREATE_INFO_KHR,
    ) -> None:
        super().__init__(
            system_id=system_id,
            create_flags=VulkanInstanceCreateFlagsKHR(create_flags).value,
            pfn_get_instance_proc_addr=pfn_get_instance_proc_addr,
            vulkan_create_info=vulkan_create_info,
            vulkan_allocator=vulkan_allocator,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.VulkanInstanceCreateInfoKHR(system_id={repr(self.system_id)}, create_flags={repr(self.create_flags)}, pfn_get_instance_proc_addr={repr(self.pfn_get_instance_proc_addr)}, vulkan_create_info={repr(self.vulkan_create_info)}, vulkan_allocator={repr(self.vulkan_allocator)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.VulkanInstanceCreateInfoKHR(system_id={self.system_id}, create_flags={self.create_flags}, pfn_get_instance_proc_addr={self.pfn_get_instance_proc_addr}, vulkan_create_info={self.vulkan_create_info}, vulkan_allocator={self.vulkan_allocator}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("system_id", SystemId),
        ("create_flags", VulkanInstanceCreateFlagsKHRCInt),
        ("pfn_get_instance_proc_addr", c_int),
        ("vulkan_create_info", POINTER(c_int)),
        ("vulkan_allocator", POINTER(c_int)),
    ]


class VulkanDeviceCreateInfoKHR(Structure):
    def __init__(
        self,
        system_id: SystemId = 0,
        create_flags: VulkanDeviceCreateFlagsKHR = VulkanDeviceCreateFlagsKHR(),
        pfn_get_instance_proc_addr: int = 0,
        vulkan_physical_device: int = 0,
        vulkan_create_info: POINTER(c_int) = None,
        vulkan_allocator: POINTER(c_int) = None,
        next: c_void_p = None,
        type: StructureType = StructureType.VULKAN_DEVICE_CREATE_INFO_KHR,
    ) -> None:
        super().__init__(
            system_id=system_id,
            create_flags=VulkanDeviceCreateFlagsKHR(create_flags).value,
            pfn_get_instance_proc_addr=pfn_get_instance_proc_addr,
            vulkan_physical_device=vulkan_physical_device,
            vulkan_create_info=vulkan_create_info,
            vulkan_allocator=vulkan_allocator,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.VulkanDeviceCreateInfoKHR(system_id={repr(self.system_id)}, create_flags={repr(self.create_flags)}, pfn_get_instance_proc_addr={repr(self.pfn_get_instance_proc_addr)}, vulkan_physical_device={repr(self.vulkan_physical_device)}, vulkan_create_info={repr(self.vulkan_create_info)}, vulkan_allocator={repr(self.vulkan_allocator)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.VulkanDeviceCreateInfoKHR(system_id={self.system_id}, create_flags={self.create_flags}, pfn_get_instance_proc_addr={self.pfn_get_instance_proc_addr}, vulkan_physical_device={self.vulkan_physical_device}, vulkan_create_info={self.vulkan_create_info}, vulkan_allocator={self.vulkan_allocator}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("system_id", SystemId),
        ("create_flags", VulkanDeviceCreateFlagsKHRCInt),
        ("pfn_get_instance_proc_addr", c_int),
        ("vulkan_physical_device", c_int),
        ("vulkan_create_info", POINTER(c_int)),
        ("vulkan_allocator", POINTER(c_int)),
    ]


GraphicsBindingVulkan2KHR = GraphicsBindingVulkanKHR


class VulkanGraphicsDeviceGetInfoKHR(Structure):
    def __init__(
        self,
        system_id: SystemId = 0,
        vulkan_instance: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.VULKAN_GRAPHICS_DEVICE_GET_INFO_KHR,
    ) -> None:
        super().__init__(
            system_id=system_id,
            vulkan_instance=vulkan_instance,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.VulkanGraphicsDeviceGetInfoKHR(system_id={repr(self.system_id)}, vulkan_instance={repr(self.vulkan_instance)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.VulkanGraphicsDeviceGetInfoKHR(system_id={self.system_id}, vulkan_instance={self.vulkan_instance}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("system_id", SystemId),
        ("vulkan_instance", c_int),
    ]


SwapchainImageVulkan2KHR = SwapchainImageVulkanKHR

GraphicsRequirementsVulkan2KHR = GraphicsRequirementsVulkanKHR

PFN_xrCreateVulkanInstanceKHR = CFUNCTYPE(Result.ctype(), Instance, POINTER(VulkanInstanceCreateInfoKHR), POINTER(c_int), POINTER(c_int))

PFN_xrCreateVulkanDeviceKHR = CFUNCTYPE(Result.ctype(), Instance, POINTER(VulkanDeviceCreateInfoKHR), POINTER(c_int), POINTER(c_int))

PFN_xrGetVulkanGraphicsDevice2KHR = CFUNCTYPE(Result.ctype(), Instance, POINTER(VulkanGraphicsDeviceGetInfoKHR), POINTER(c_int))

PFN_xrGetVulkanGraphicsRequirements2KHR = CFUNCTYPE(Result.ctype(), Instance, SystemId, POINTER(GraphicsRequirementsVulkanKHR))


class SwapchainImageFoveationVulkanFB(Structure):
    def __init__(
        self,
        image: int = 0,
        width: int = 0,
        height: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.SWAPCHAIN_IMAGE_FOVEATION_VULKAN_FB,
    ) -> None:
        super().__init__(
            image=image,
            width=width,
            height=height,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.SwapchainImageFoveationVulkanFB(image={repr(self.image)}, width={repr(self.width)}, height={repr(self.height)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.SwapchainImageFoveationVulkanFB(image={self.image}, width={self.width}, height={self.height}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("image", c_int),
        ("width", c_uint32),
        ("height", c_uint32),
    ]


class SwapchainStateSamplerVulkanFB(Structure):
    def __init__(
        self,
        min_filter: int = 0,
        mag_filter: int = 0,
        mipmap_mode: int = 0,
        wrap_mode_s: int = 0,
        wrap_mode_t: int = 0,
        swizzle_red: int = 0,
        swizzle_green: int = 0,
        swizzle_blue: int = 0,
        swizzle_alpha: int = 0,
        max_anisotropy: float = 0,
        border_color: Color4f = None,
        next: c_void_p = None,
        type: StructureType = StructureType.SWAPCHAIN_STATE_SAMPLER_VULKAN_FB,
    ) -> None:
        if border_color is None:
            border_color = Color4f()
        super().__init__(
            min_filter=min_filter,
            mag_filter=mag_filter,
            mipmap_mode=mipmap_mode,
            wrap_mode_s=wrap_mode_s,
            wrap_mode_t=wrap_mode_t,
            swizzle_red=swizzle_red,
            swizzle_green=swizzle_green,
            swizzle_blue=swizzle_blue,
            swizzle_alpha=swizzle_alpha,
            max_anisotropy=max_anisotropy,
            border_color=border_color,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.SwapchainStateSamplerVulkanFB(min_filter={repr(self.min_filter)}, mag_filter={repr(self.mag_filter)}, mipmap_mode={repr(self.mipmap_mode)}, wrap_mode_s={repr(self.wrap_mode_s)}, wrap_mode_t={repr(self.wrap_mode_t)}, swizzle_red={repr(self.swizzle_red)}, swizzle_green={repr(self.swizzle_green)}, swizzle_blue={repr(self.swizzle_blue)}, swizzle_alpha={repr(self.swizzle_alpha)}, max_anisotropy={repr(self.max_anisotropy)}, border_color={repr(self.border_color)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.SwapchainStateSamplerVulkanFB(min_filter={self.min_filter}, mag_filter={self.mag_filter}, mipmap_mode={self.mipmap_mode}, wrap_mode_s={self.wrap_mode_s}, wrap_mode_t={self.wrap_mode_t}, swizzle_red={self.swizzle_red}, swizzle_green={self.swizzle_green}, swizzle_blue={self.swizzle_blue}, swizzle_alpha={self.swizzle_alpha}, max_anisotropy={self.max_anisotropy:.3f}, border_color={self.border_color}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("min_filter", c_int),
        ("mag_filter", c_int),
        ("mipmap_mode", c_int),
        ("wrap_mode_s", c_int),
        ("wrap_mode_t", c_int),
        ("swizzle_red", c_int),
        ("swizzle_green", c_int),
        ("swizzle_blue", c_int),
        ("swizzle_alpha", c_int),
        ("max_anisotropy", c_float),
        ("border_color", Color4f),
    ]


class VulkanSwapchainCreateInfoMETA(Structure):
    def __init__(
        self,
        additional_create_flags: int = 0,
        additional_usage_flags: int = 0,
        next: c_void_p = None,
        type: StructureType = StructureType.VULKAN_SWAPCHAIN_CREATE_INFO_META,
    ) -> None:
        super().__init__(
            additional_create_flags=additional_create_flags,
            additional_usage_flags=additional_usage_flags,
            next=next,
            type=type,
        )

    def __repr__(self) -> str:
        return f"xr.VulkanSwapchainCreateInfoMETA(additional_create_flags={repr(self.additional_create_flags)}, additional_usage_flags={repr(self.additional_usage_flags)}, next={repr(self.next)}, type={repr(self.type)})"

    def __str__(self) -> str:
        return f"xr.VulkanSwapchainCreateInfoMETA(additional_create_flags={self.additional_create_flags}, additional_usage_flags={self.additional_usage_flags}, next={self.next}, type={self.type})"

    _fields_ = [
        ("type", StructureType.ctype()),
        ("next", c_void_p),
        ("additional_create_flags", c_int),
        ("additional_usage_flags", c_int),
    ]


__all__ = [
    "FB_FOVEATION_VULKAN_EXTENSION_NAME",
    "FB_SWAPCHAIN_UPDATE_STATE_VULKAN_EXTENSION_NAME",
    "FB_foveation_vulkan",
    "FB_foveation_vulkan_SPEC_VERSION",
    "FB_swapchain_update_state_vulkan",
    "FB_swapchain_update_state_vulkan_SPEC_VERSION",
    "GraphicsBindingOpenGLWaylandKHR",
    "GraphicsBindingOpenGLXcbKHR",
    "GraphicsBindingOpenGLXlibKHR",
    "GraphicsBindingVulkan2KHR",
    "GraphicsBindingVulkanKHR",
    "GraphicsRequirementsOpenGLKHR",
    "GraphicsRequirementsVulkan2KHR",
    "GraphicsRequirementsVulkanKHR",
    "KHR_CONVERT_TIMESPEC_TIME_EXTENSION_NAME",
    "KHR_OPENGL_ENABLE_EXTENSION_NAME",
    "KHR_VULKAN_ENABLE2_EXTENSION_NAME",
    "KHR_VULKAN_ENABLE_EXTENSION_NAME",
    "KHR_VULKAN_SWAPCHAIN_FORMAT_LIST_EXTENSION_NAME",
    "KHR_convert_timespec_time",
    "KHR_convert_timespec_time_SPEC_VERSION",
    "KHR_opengl_enable",
    "KHR_opengl_enable_SPEC_VERSION",
    "KHR_vulkan_enable",
    "KHR_vulkan_enable2",
    "KHR_vulkan_enable2_SPEC_VERSION",
    "KHR_vulkan_enable_SPEC_VERSION",
    "KHR_vulkan_swapchain_format_list",
    "KHR_vulkan_swapchain_format_list_SPEC_VERSION",
    "META_VULKAN_SWAPCHAIN_CREATE_INFO_EXTENSION_NAME",
    "META_vulkan_swapchain_create_info",
    "META_vulkan_swapchain_create_info_SPEC_VERSION",
    "PFN_xrConvertTimeToTimespecTimeKHR",
    "PFN_xrConvertTimespecTimeToTimeKHR",
    "PFN_xrCreateVulkanDeviceKHR",
    "PFN_xrCreateVulkanInstanceKHR",
    "PFN_xrGetOpenGLGraphicsRequirementsKHR",
    "PFN_xrGetVulkanDeviceExtensionsKHR",
    "PFN_xrGetVulkanGraphicsDevice2KHR",
    "PFN_xrGetVulkanGraphicsDeviceKHR",
    "PFN_xrGetVulkanGraphicsRequirements2KHR",
    "PFN_xrGetVulkanGraphicsRequirementsKHR",
    "PFN_xrGetVulkanInstanceExtensionsKHR",
    "SwapchainImageFoveationVulkanFB",
    "SwapchainImageOpenGLKHR",
    "SwapchainImageVulkan2KHR",
    "SwapchainImageVulkanKHR",
    "SwapchainStateSamplerVulkanFB",
    "VulkanDeviceCreateFlagsKHR",
    "VulkanDeviceCreateFlagsKHRCInt",
    "VulkanDeviceCreateInfoKHR",
    "VulkanGraphicsDeviceGetInfoKHR",
    "VulkanInstanceCreateFlagsKHR",
    "VulkanInstanceCreateFlagsKHRCInt",
    "VulkanInstanceCreateInfoKHR",
    "VulkanSwapchainCreateInfoMETA",
    "VulkanSwapchainFormatListCreateInfoKHR",
]



from typing import Optional
from .enums import Result

# raise_on_qualified_success is a module setting to control whether positive non-SUCCESS result
# trigger exceptions.
raise_on_qualified_success = True


class XrException(Exception):
    """Base class for all OpenXR exceptions."""

    @staticmethod
    def is_exception() -> bool:
        return True


class ResultException(XrException):
    """Exception related to return value of and OpenXR function."""

    @staticmethod
    def get_result_enum() -> Optional[Result]:
        return None

    @staticmethod
    def is_exception() -> bool:
        return False


class ErrorResult(ResultException):
    """Error during OpenXR function call."""

    @staticmethod
    def is_exception() -> bool:
        return True


class QualifiedSuccessResult(ResultException):
    """An OpenXR function returned a non-error status other than SUCCESS"""

    @staticmethod
    def is_exception() -> bool:
        return raise_on_qualified_success


class Success(ResultException):
    """Function successfully completed."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "Function successfully completed."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.SUCCESS


class TimeoutExpired(QualifiedSuccessResult):
    """The specified timeout time occurred before the operation could complete."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified timeout time occurred before the operation could complete."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.TIMEOUT_EXPIRED


class SessionLossPending(QualifiedSuccessResult):
    """The session will be lost soon."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session will be lost soon."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.SESSION_LOSS_PENDING


class EventUnavailable(QualifiedSuccessResult):
    """No event was available."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "No event was available."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.EVENT_UNAVAILABLE


class SpaceBoundsUnavailable(QualifiedSuccessResult):
    """The space's bounds are not known at the moment."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The space's bounds are not known at the moment."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.SPACE_BOUNDS_UNAVAILABLE


class SessionNotFocused(QualifiedSuccessResult):
    """The session is not in the focused state."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session is not in the focused state."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.SESSION_NOT_FOCUSED


class FrameDiscarded(QualifiedSuccessResult):
    """A frame has been discarded from composition."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A frame has been discarded from composition."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.FRAME_DISCARDED


class ValidationFailureError(ErrorResult):
    """The function usage was invalid in some way."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The function usage was invalid in some way."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_VALIDATION_FAILURE


class RuntimeFailureError(ErrorResult):
    """The runtime failed to handle the function in an unexpected way that is not covered by another error result. """

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The runtime failed to handle the function in an unexpected way that is not covered by another error result. "
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_RUNTIME_FAILURE


class OutOfMemoryError(ErrorResult):
    """A memory allocation has failed."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A memory allocation has failed."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_OUT_OF_MEMORY


class ApiVersionUnsupportedError(ErrorResult):
    """The runtime does not support the requested API version."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The runtime does not support the requested API version."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_API_VERSION_UNSUPPORTED


class InitializationFailedError(ErrorResult):
    """Initialization of object could not be completed."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "Initialization of object could not be completed."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_INITIALIZATION_FAILED


class FunctionUnsupportedError(ErrorResult):
    """The requested function was not found or is otherwise unsupported."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The requested function was not found or is otherwise unsupported."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FUNCTION_UNSUPPORTED


class FeatureUnsupportedError(ErrorResult):
    """The requested feature is not supported."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The requested feature is not supported."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FEATURE_UNSUPPORTED


class ExtensionNotPresentError(ErrorResult):
    """A requested extension is not supported."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A requested extension is not supported."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_EXTENSION_NOT_PRESENT


class LimitReachedError(ErrorResult):
    """The runtime supports no more of the requested resource."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The runtime supports no more of the requested resource."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_LIMIT_REACHED


class SizeInsufficientError(ErrorResult):
    """The supplied size was smaller than required."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The supplied size was smaller than required."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SIZE_INSUFFICIENT


class HandleInvalidError(ErrorResult):
    """A supplied object handle was invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A supplied object handle was invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_HANDLE_INVALID


class InstanceLostError(ErrorResult):
    """The slink:XrInstance was lost or could not be found. It will need to be destroyed and optionally recreated."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The slink:XrInstance was lost or could not be found. It will need to be destroyed and optionally recreated."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_INSTANCE_LOST


class SessionRunningError(ErrorResult):
    """The session <<session_running, is already running>>."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session <<session_running, is already running>>."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SESSION_RUNNING


class SessionNotRunningError(ErrorResult):
    """The session <<session_not_running, is not yet running>>."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session <<session_not_running, is not yet running>>."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SESSION_NOT_RUNNING


class SessionLostError(ErrorResult):
    """The slink:XrSession was lost. It will need to be destroyed and optionally recreated."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The slink:XrSession was lost. It will need to be destroyed and optionally recreated."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SESSION_LOST


class SystemInvalidError(ErrorResult):
    """The provided basetype:XrSystemId was invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The provided basetype:XrSystemId was invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SYSTEM_INVALID


class PathInvalidError(ErrorResult):
    """The provided basetype:XrPath was not valid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The provided basetype:XrPath was not valid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_PATH_INVALID


class PathCountExceededError(ErrorResult):
    """The maximum number of supported semantic paths has been reached."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The maximum number of supported semantic paths has been reached."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_PATH_COUNT_EXCEEDED


class PathFormatInvalidError(ErrorResult):
    """The semantic path character format is invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The semantic path character format is invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_PATH_FORMAT_INVALID


class PathUnsupportedError(ErrorResult):
    """The semantic path is unsupported."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The semantic path is unsupported."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_PATH_UNSUPPORTED


class LayerInvalidError(ErrorResult):
    """The layer was NULL or otherwise invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The layer was NULL or otherwise invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_LAYER_INVALID


class LayerLimitExceededError(ErrorResult):
    """The number of specified layers is greater than the supported number."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The number of specified layers is greater than the supported number."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_LAYER_LIMIT_EXCEEDED


class SwapchainRectInvalidError(ErrorResult):
    """The image rect was negatively sized or otherwise invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The image rect was negatively sized or otherwise invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SWAPCHAIN_RECT_INVALID


class SwapchainFormatUnsupportedError(ErrorResult):
    """The image format is not supported by the runtime or platform."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The image format is not supported by the runtime or platform."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SWAPCHAIN_FORMAT_UNSUPPORTED


class ActionTypeMismatchError(ErrorResult):
    """The API used to retrieve an action's state does not match the action's type."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The API used to retrieve an action's state does not match the action's type."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ACTION_TYPE_MISMATCH


class SessionNotReadyError(ErrorResult):
    """The session is not in the ready state."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session is not in the ready state."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SESSION_NOT_READY


class SessionNotStoppingError(ErrorResult):
    """The session is not in the stopping state."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session is not in the stopping state."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SESSION_NOT_STOPPING


class TimeInvalidError(ErrorResult):
    """The provided basetype:XrTime was zero, negative, or out of range."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The provided basetype:XrTime was zero, negative, or out of range."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_TIME_INVALID


class ReferenceSpaceUnsupportedError(ErrorResult):
    """The specified reference space is not supported by the runtime or system."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified reference space is not supported by the runtime or system."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_REFERENCE_SPACE_UNSUPPORTED


class FileAccessErrorError(ErrorResult):
    """The file could not be accessed."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The file could not be accessed."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FILE_ACCESS_ERROR


class FileContentsInvalidError(ErrorResult):
    """The file's contents were invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The file's contents were invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FILE_CONTENTS_INVALID


class FormFactorUnsupportedError(ErrorResult):
    """The specified form factor is not supported by the current runtime or platform."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified form factor is not supported by the current runtime or platform."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FORM_FACTOR_UNSUPPORTED


class FormFactorUnavailableError(ErrorResult):
    """The specified form factor is supported, but the device is currently not available, e.g. not plugged in or powered off."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified form factor is supported, but the device is currently not available, e.g. not plugged in or powered off."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FORM_FACTOR_UNAVAILABLE


class ApiLayerNotPresentError(ErrorResult):
    """A requested API layer is not present or could not be loaded."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A requested API layer is not present or could not be loaded."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_API_LAYER_NOT_PRESENT


class CallOrderInvalidError(ErrorResult):
    """The call was made without having made a previously required call."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The call was made without having made a previously required call."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_CALL_ORDER_INVALID


class GraphicsDeviceInvalidError(ErrorResult):
    """The given graphics device is not in a valid state. The graphics device could be lost or initialized without meeting graphics requirements."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The given graphics device is not in a valid state. The graphics device could be lost or initialized without meeting graphics requirements."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_GRAPHICS_DEVICE_INVALID


class PoseInvalidError(ErrorResult):
    """The supplied pose was invalid with respect to the requirements."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The supplied pose was invalid with respect to the requirements."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_POSE_INVALID


class IndexOutOfRangeError(ErrorResult):
    """The supplied index was outside the range of valid indices."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The supplied index was outside the range of valid indices."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_INDEX_OUT_OF_RANGE


class ViewConfigurationTypeUnsupportedError(ErrorResult):
    """The specified view configuration type is not supported by the runtime or platform."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified view configuration type is not supported by the runtime or platform."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_VIEW_CONFIGURATION_TYPE_UNSUPPORTED


class EnvironmentBlendModeUnsupportedError(ErrorResult):
    """The specified environment blend mode is not supported by the runtime or platform."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The specified environment blend mode is not supported by the runtime or platform."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ENVIRONMENT_BLEND_MODE_UNSUPPORTED


class NameDuplicatedError(ErrorResult):
    """The name provided was a duplicate of an already-existing resource."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The name provided was a duplicate of an already-existing resource."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_NAME_DUPLICATED


class NameInvalidError(ErrorResult):
    """The name provided was invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The name provided was invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_NAME_INVALID


class ActionsetNotAttachedError(ErrorResult):
    """A referenced action set is not attached to the session."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "A referenced action set is not attached to the session."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ACTIONSET_NOT_ATTACHED


class ActionsetsAlreadyAttachedError(ErrorResult):
    """The session already has attached action sets."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The session already has attached action sets."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ACTIONSETS_ALREADY_ATTACHED


class LocalizedNameDuplicatedError(ErrorResult):
    """The localized name provided was a duplicate of an already-existing resource."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The localized name provided was a duplicate of an already-existing resource."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_LOCALIZED_NAME_DUPLICATED


class LocalizedNameInvalidError(ErrorResult):
    """The localized name provided was invalid."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The localized name provided was invalid."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_LOCALIZED_NAME_INVALID


class GraphicsRequirementsCallMissingError(ErrorResult):
    """The fname:xrGetGraphicsRequirements* call was not made before calling fname:xrCreateSession."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The fname:xrGetGraphicsRequirements* call was not made before calling fname:xrCreateSession."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_GRAPHICS_REQUIREMENTS_CALL_MISSING


class RuntimeUnavailableError(ErrorResult):
    """The loader was unable to find or load a runtime."""

    def __init__(self, message=None):
        if message is None:
            super().__init__(
                "The loader was unable to find or load a runtime."
            )
        else:
            super().__init__(message)

    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_RUNTIME_UNAVAILABLE


class AndroidThreadSettingsIdInvalidKHRError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ANDROID_THREAD_SETTINGS_ID_INVALID_KHR


class AndroidThreadSettingsFailureKHRError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_ANDROID_THREAD_SETTINGS_FAILURE_KHR


class CreateSpatialAnchorFailedMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_CREATE_SPATIAL_ANCHOR_FAILED_MSFT


class SecondaryViewConfigurationTypeNotEnabledMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SECONDARY_VIEW_CONFIGURATION_TYPE_NOT_ENABLED_MSFT


class ControllerModelKeyInvalidMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_CONTROLLER_MODEL_KEY_INVALID_MSFT


class ReprojectionModeUnsupportedMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_REPROJECTION_MODE_UNSUPPORTED_MSFT


class ComputeNewSceneNotCompletedMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_COMPUTE_NEW_SCENE_NOT_COMPLETED_MSFT


class SceneComponentIdInvalidMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SCENE_COMPONENT_ID_INVALID_MSFT


class SceneComponentTypeMismatchMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SCENE_COMPONENT_TYPE_MISMATCH_MSFT


class SceneMeshBufferIdInvalidMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SCENE_MESH_BUFFER_ID_INVALID_MSFT


class SceneComputeFeatureIncompatibleMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SCENE_COMPUTE_FEATURE_INCOMPATIBLE_MSFT


class SceneComputeConsistencyMismatchMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SCENE_COMPUTE_CONSISTENCY_MISMATCH_MSFT


class DisplayRefreshRateUnsupportedFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_DISPLAY_REFRESH_RATE_UNSUPPORTED_FB


class ColorSpaceUnsupportedFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_COLOR_SPACE_UNSUPPORTED_FB


class SpaceComponentNotSupportedFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPACE_COMPONENT_NOT_SUPPORTED_FB


class SpaceComponentNotEnabledFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPACE_COMPONENT_NOT_ENABLED_FB


class SpaceComponentStatusPendingFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPACE_COMPONENT_STATUS_PENDING_FB


class SpaceComponentStatusAlreadySetFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPACE_COMPONENT_STATUS_ALREADY_SET_FB


class UnexpectedStatePassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_UNEXPECTED_STATE_PASSTHROUGH_FB


class FeatureAlreadyCreatedPassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FEATURE_ALREADY_CREATED_PASSTHROUGH_FB


class FeatureRequiredPassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_FEATURE_REQUIRED_PASSTHROUGH_FB


class NotPermittedPassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_NOT_PERMITTED_PASSTHROUGH_FB


class InsufficientResourcesPassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_INSUFFICIENT_RESOURCES_PASSTHROUGH_FB


class UnknownPassthroughFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_UNKNOWN_PASSTHROUGH_FB


class RenderModelKeyInvalidFBError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_RENDER_MODEL_KEY_INVALID_FB


class RenderModelUnavailableFB(QualifiedSuccessResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.RENDER_MODEL_UNAVAILABLE_FB


class MarkerNotTrackedVarjoError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_MARKER_NOT_TRACKED_VARJO


class MarkerIdInvalidVarjoError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_MARKER_ID_INVALID_VARJO


class SpatialAnchorNameNotFoundMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPATIAL_ANCHOR_NAME_NOT_FOUND_MSFT


class SpatialAnchorNameInvalidMSFTError(ErrorResult):
    @staticmethod
    def get_result_enum() -> Result:
        return Result.ERROR_SPATIAL_ANCHOR_NAME_INVALID_MSFT


_exception_map = {
    Result.SUCCESS: Success,
    Result.TIMEOUT_EXPIRED: TimeoutExpired,
    Result.SESSION_LOSS_PENDING: SessionLossPending,
    Result.EVENT_UNAVAILABLE: EventUnavailable,
    Result.SPACE_BOUNDS_UNAVAILABLE: SpaceBoundsUnavailable,
    Result.SESSION_NOT_FOCUSED: SessionNotFocused,
    Result.FRAME_DISCARDED: FrameDiscarded,
    Result.ERROR_VALIDATION_FAILURE: ValidationFailureError,
    Result.ERROR_RUNTIME_FAILURE: RuntimeFailureError,
    Result.ERROR_OUT_OF_MEMORY: OutOfMemoryError,
    Result.ERROR_API_VERSION_UNSUPPORTED: ApiVersionUnsupportedError,
    Result.ERROR_INITIALIZATION_FAILED: InitializationFailedError,
    Result.ERROR_FUNCTION_UNSUPPORTED: FunctionUnsupportedError,
    Result.ERROR_FEATURE_UNSUPPORTED: FeatureUnsupportedError,
    Result.ERROR_EXTENSION_NOT_PRESENT: ExtensionNotPresentError,
    Result.ERROR_LIMIT_REACHED: LimitReachedError,
    Result.ERROR_SIZE_INSUFFICIENT: SizeInsufficientError,
    Result.ERROR_HANDLE_INVALID: HandleInvalidError,
    Result.ERROR_INSTANCE_LOST: InstanceLostError,
    Result.ERROR_SESSION_RUNNING: SessionRunningError,
    Result.ERROR_SESSION_NOT_RUNNING: SessionNotRunningError,
    Result.ERROR_SESSION_LOST: SessionLostError,
    Result.ERROR_SYSTEM_INVALID: SystemInvalidError,
    Result.ERROR_PATH_INVALID: PathInvalidError,
    Result.ERROR_PATH_COUNT_EXCEEDED: PathCountExceededError,
    Result.ERROR_PATH_FORMAT_INVALID: PathFormatInvalidError,
    Result.ERROR_PATH_UNSUPPORTED: PathUnsupportedError,
    Result.ERROR_LAYER_INVALID: LayerInvalidError,
    Result.ERROR_LAYER_LIMIT_EXCEEDED: LayerLimitExceededError,
    Result.ERROR_SWAPCHAIN_RECT_INVALID: SwapchainRectInvalidError,
    Result.ERROR_SWAPCHAIN_FORMAT_UNSUPPORTED: SwapchainFormatUnsupportedError,
    Result.ERROR_ACTION_TYPE_MISMATCH: ActionTypeMismatchError,
    Result.ERROR_SESSION_NOT_READY: SessionNotReadyError,
    Result.ERROR_SESSION_NOT_STOPPING: SessionNotStoppingError,
    Result.ERROR_TIME_INVALID: TimeInvalidError,
    Result.ERROR_REFERENCE_SPACE_UNSUPPORTED: ReferenceSpaceUnsupportedError,
    Result.ERROR_FILE_ACCESS_ERROR: FileAccessErrorError,
    Result.ERROR_FILE_CONTENTS_INVALID: FileContentsInvalidError,
    Result.ERROR_FORM_FACTOR_UNSUPPORTED: FormFactorUnsupportedError,
    Result.ERROR_FORM_FACTOR_UNAVAILABLE: FormFactorUnavailableError,
    Result.ERROR_API_LAYER_NOT_PRESENT: ApiLayerNotPresentError,
    Result.ERROR_CALL_ORDER_INVALID: CallOrderInvalidError,
    Result.ERROR_GRAPHICS_DEVICE_INVALID: GraphicsDeviceInvalidError,
    Result.ERROR_POSE_INVALID: PoseInvalidError,
    Result.ERROR_INDEX_OUT_OF_RANGE: IndexOutOfRangeError,
    Result.ERROR_VIEW_CONFIGURATION_TYPE_UNSUPPORTED: ViewConfigurationTypeUnsupportedError,
    Result.ERROR_ENVIRONMENT_BLEND_MODE_UNSUPPORTED: EnvironmentBlendModeUnsupportedError,
    Result.ERROR_NAME_DUPLICATED: NameDuplicatedError,
    Result.ERROR_NAME_INVALID: NameInvalidError,
    Result.ERROR_ACTIONSET_NOT_ATTACHED: ActionsetNotAttachedError,
    Result.ERROR_ACTIONSETS_ALREADY_ATTACHED: ActionsetsAlreadyAttachedError,
    Result.ERROR_LOCALIZED_NAME_DUPLICATED: LocalizedNameDuplicatedError,
    Result.ERROR_LOCALIZED_NAME_INVALID: LocalizedNameInvalidError,
    Result.ERROR_GRAPHICS_REQUIREMENTS_CALL_MISSING: GraphicsRequirementsCallMissingError,
    Result.ERROR_RUNTIME_UNAVAILABLE: RuntimeUnavailableError,
    Result.ERROR_ANDROID_THREAD_SETTINGS_ID_INVALID_KHR: AndroidThreadSettingsIdInvalidKHRError,
    Result.ERROR_ANDROID_THREAD_SETTINGS_FAILURE_KHR: AndroidThreadSettingsFailureKHRError,
    Result.ERROR_CREATE_SPATIAL_ANCHOR_FAILED_MSFT: CreateSpatialAnchorFailedMSFTError,
    Result.ERROR_SECONDARY_VIEW_CONFIGURATION_TYPE_NOT_ENABLED_MSFT: SecondaryViewConfigurationTypeNotEnabledMSFTError,
    Result.ERROR_CONTROLLER_MODEL_KEY_INVALID_MSFT: ControllerModelKeyInvalidMSFTError,
    Result.ERROR_REPROJECTION_MODE_UNSUPPORTED_MSFT: ReprojectionModeUnsupportedMSFTError,
    Result.ERROR_COMPUTE_NEW_SCENE_NOT_COMPLETED_MSFT: ComputeNewSceneNotCompletedMSFTError,
    Result.ERROR_SCENE_COMPONENT_ID_INVALID_MSFT: SceneComponentIdInvalidMSFTError,
    Result.ERROR_SCENE_COMPONENT_TYPE_MISMATCH_MSFT: SceneComponentTypeMismatchMSFTError,
    Result.ERROR_SCENE_MESH_BUFFER_ID_INVALID_MSFT: SceneMeshBufferIdInvalidMSFTError,
    Result.ERROR_SCENE_COMPUTE_FEATURE_INCOMPATIBLE_MSFT: SceneComputeFeatureIncompatibleMSFTError,
    Result.ERROR_SCENE_COMPUTE_CONSISTENCY_MISMATCH_MSFT: SceneComputeConsistencyMismatchMSFTError,
    Result.ERROR_DISPLAY_REFRESH_RATE_UNSUPPORTED_FB: DisplayRefreshRateUnsupportedFBError,
    Result.ERROR_COLOR_SPACE_UNSUPPORTED_FB: ColorSpaceUnsupportedFBError,
    Result.ERROR_SPACE_COMPONENT_NOT_SUPPORTED_FB: SpaceComponentNotSupportedFBError,
    Result.ERROR_SPACE_COMPONENT_NOT_ENABLED_FB: SpaceComponentNotEnabledFBError,
    Result.ERROR_SPACE_COMPONENT_STATUS_PENDING_FB: SpaceComponentStatusPendingFBError,
    Result.ERROR_SPACE_COMPONENT_STATUS_ALREADY_SET_FB: SpaceComponentStatusAlreadySetFBError,
    Result.ERROR_UNEXPECTED_STATE_PASSTHROUGH_FB: UnexpectedStatePassthroughFBError,
    Result.ERROR_FEATURE_ALREADY_CREATED_PASSTHROUGH_FB: FeatureAlreadyCreatedPassthroughFBError,
    Result.ERROR_FEATURE_REQUIRED_PASSTHROUGH_FB: FeatureRequiredPassthroughFBError,
    Result.ERROR_NOT_PERMITTED_PASSTHROUGH_FB: NotPermittedPassthroughFBError,
    Result.ERROR_INSUFFICIENT_RESOURCES_PASSTHROUGH_FB: InsufficientResourcesPassthroughFBError,
    Result.ERROR_UNKNOWN_PASSTHROUGH_FB: UnknownPassthroughFBError,
    Result.ERROR_RENDER_MODEL_KEY_INVALID_FB: RenderModelKeyInvalidFBError,
    Result.RENDER_MODEL_UNAVAILABLE_FB: RenderModelUnavailableFB,
    Result.ERROR_MARKER_NOT_TRACKED_VARJO: MarkerNotTrackedVarjoError,
    Result.ERROR_MARKER_ID_INVALID_VARJO: MarkerIdInvalidVarjoError,
    Result.ERROR_SPATIAL_ANCHOR_NAME_NOT_FOUND_MSFT: SpatialAnchorNameNotFoundMSFTError,
    Result.ERROR_SPATIAL_ANCHOR_NAME_INVALID_MSFT: SpatialAnchorNameInvalidMSFTError,
}


def check_result(
    xr_result: Result, message: str = None
) -> XrException:
    if xr_result in _exception_map:
        xr_result_exception = _exception_map[xr_result]
    else:
        if xr_result.value < 0:
            xr_result_exception = ErrorResult
        elif xr_result.value > 1:
            xr_result_exception = QualifiedSuccessResult
        else:
            xr_result_exception = Success
    if message is None:
        # TODO: I see a message in the logging...
        return xr_result_exception()
    else:
        return xr_result_exception(message)


__all__ = [
    "ActionTypeMismatchError",
    "ActionsetNotAttachedError",
    "ActionsetsAlreadyAttachedError",
    "AndroidThreadSettingsFailureKHRError",
    "AndroidThreadSettingsIdInvalidKHRError",
    "ApiLayerNotPresentError",
    "ApiVersionUnsupportedError",
    "CallOrderInvalidError",
    "ColorSpaceUnsupportedFBError",
    "ComputeNewSceneNotCompletedMSFTError",
    "ControllerModelKeyInvalidMSFTError",
    "CreateSpatialAnchorFailedMSFTError",
    "DisplayRefreshRateUnsupportedFBError",
    "EnvironmentBlendModeUnsupportedError",
    "ErrorResult",
    "EventUnavailable",
    "ExtensionNotPresentError",
    "FeatureAlreadyCreatedPassthroughFBError",
    "FeatureRequiredPassthroughFBError",
    "FeatureUnsupportedError",
    "FileAccessErrorError",
    "FileContentsInvalidError",
    "FormFactorUnavailableError",
    "FormFactorUnsupportedError",
    "FrameDiscarded",
    "FunctionUnsupportedError",
    "GraphicsDeviceInvalidError",
    "GraphicsRequirementsCallMissingError",
    "HandleInvalidError",
    "IndexOutOfRangeError",
    "InitializationFailedError",
    "InstanceLostError",
    "InsufficientResourcesPassthroughFBError",
    "LayerInvalidError",
    "LayerLimitExceededError",
    "LimitReachedError",
    "LocalizedNameDuplicatedError",
    "LocalizedNameInvalidError",
    "MarkerIdInvalidVarjoError",
    "MarkerNotTrackedVarjoError",
    "NameDuplicatedError",
    "NameInvalidError",
    "NotPermittedPassthroughFBError",
    "OutOfMemoryError",
    "PathCountExceededError",
    "PathFormatInvalidError",
    "PathInvalidError",
    "PathUnsupportedError",
    "PoseInvalidError",
    "QualifiedSuccessResult",
    "ReferenceSpaceUnsupportedError",
    "RenderModelKeyInvalidFBError",
    "RenderModelUnavailableFB",
    "ReprojectionModeUnsupportedMSFTError",
    "ResultException",
    "RuntimeFailureError",
    "RuntimeUnavailableError",
    "SceneComponentIdInvalidMSFTError",
    "SceneComponentTypeMismatchMSFTError",
    "SceneComputeConsistencyMismatchMSFTError",
    "SceneComputeFeatureIncompatibleMSFTError",
    "SceneMeshBufferIdInvalidMSFTError",
    "SecondaryViewConfigurationTypeNotEnabledMSFTError",
    "SessionLossPending",
    "SessionLostError",
    "SessionNotFocused",
    "SessionNotReadyError",
    "SessionNotRunningError",
    "SessionNotStoppingError",
    "SessionRunningError",
    "SizeInsufficientError",
    "SpaceBoundsUnavailable",
    "SpaceComponentNotEnabledFBError",
    "SpaceComponentNotSupportedFBError",
    "SpaceComponentStatusAlreadySetFBError",
    "SpaceComponentStatusPendingFBError",
    "SpatialAnchorNameInvalidMSFTError",
    "SpatialAnchorNameNotFoundMSFTError",
    "Success",
    "SwapchainFormatUnsupportedError",
    "SwapchainRectInvalidError",
    "SystemInvalidError",
    "TimeInvalidError",
    "TimeoutExpired",
    "UnexpectedStatePassthroughFBError",
    "UnknownPassthroughFBError",
    "ValidationFailureError",
    "ViewConfigurationTypeUnsupportedError",
    "XrException",
    "check_result",
]



"""
`xr` is the top level module of the pyopenxr unofficial python bindings for the
OpenXR SDK to access VR and AR devices.
"""

from . import (
    version,
    constants,
    enums,
    typedefs,
    functions,
    platform,
    exception,
    classes,
    api_layer,
    context_object,
    opengl_graphics,
    matrix4x4f,
    extension,
)

from .version import *
from .constants import *
from .enums import *
from .typedefs import *
from .functions import *
from .platform import *
from .exception import *
from .classes import *
from .api_layer import *
from .context_object import *
from .opengl_graphics import *
from .matrix4x4f import *


# from .experiment import *

__all__ = [
    "extension",
]

for subpackage in (
    version,
    constants,
    enums,
    typedefs,
    functions,
    platform,
    exception,
    classes,
    api_layer,
    context_object,
    opengl_graphics,
    matrix4x4f,
):
    __all__ += subpackage.__all__

from .api_layer.steamvr_linux_destroyinstance_layer import SteamVrLinuxDestroyInstanceLayer
__all__ += "SteamVrLinuxDestroyInstanceLayer"

__version__ = version.PYOPENXR_VERSION  # Not in __all__, right?



# Warning: this file is automatically generated. Do not edit.

# pyopenxr version is based on openxr version...
# except the patch number is:
#   100 * openxr patch number + pyopenxr patch number

XR_VERSION_MAJOR = 1
XR_VERSION_MINOR = 0
XR_VERSION_PATCH = 25
XR_CURRENT_API_VERSION_STRING = "1.0.25"

PYOPENXR_VERSION_MAJOR = 1
PYOPENXR_VERSION_MINOR = 0
PYOPENXR_VERSION_PATCH = 2502
PYOPENXR_VERSION_PATCH_INCREMENTAL = 2
PYOPENXR_VERSION_SUFFIX = ""
PYOPENXR_VERSION = "1.0.2502"


class Version(object):
    def __init__(self, major: int = 0, minor: int = None, patch: int = None):
        if minor is None and patch is None and major > 0xffff:
            # major argument is actually a packed xr.VersionNumber
            patch = major & 0xffffffff
            minor = (major >> 32) & 0xffff
            major = (major >> 48) & 0xffff
        if minor is None:
            minor = 0
        if patch is None:
            patch = 0
        self.major = major
        self.minor = minor
        self.patch = patch

    def __index__(self) -> int:
        """Packed xr.VersionNumber"""
        return (((int(self.major) & 0xffff) << 48) 
                | ((int(self.minor) & 0xffff) << 32) 
                | (int(self.patch) & 0xffffffff))

    def __int__(self) -> int:
        return self.__index__()

    def number(self) -> int:
        """Packed xr.VersionNumber"""
        return self.__index__()

    def __str__(self):
        return f"{self.major}.{self.minor}.{self.patch}"


XR_CURRENT_API_VERSION = Version(XR_VERSION_MAJOR, XR_VERSION_MINOR, XR_VERSION_PATCH)
PYOPENXR_CURRENT_API_VERSION = Version(
    PYOPENXR_VERSION_MAJOR,
    PYOPENXR_VERSION_MINOR,
    PYOPENXR_VERSION_PATCH
)


__version__ = PYOPENXR_VERSION

__all__ = [
    "PYOPENXR_CURRENT_API_VERSION",
    "PYOPENXR_VERSION_MAJOR",
    "PYOPENXR_VERSION_MINOR",
    "PYOPENXR_VERSION_PATCH",
    "PYOPENXR_VERSION_PATCH_INCREMENTAL",
    "PYOPENXR_VERSION_SUFFIX",
    "PYOPENXR_VERSION",
    "Version",
    "XR_CURRENT_API_VERSION",
    "XR_VERSION_MAJOR",
    "XR_VERSION_MINOR",
    "XR_VERSION_PATCH",
]



# Warning: this file is auto-generated. Do not edit.

from ctypes import Array, POINTER, byref, c_char, c_int64, c_uint32, cast, create_string_buffer

"""
File xr.functions.py

Defines high-level pythonic function definitions for pyopenxr.
"""

from . import raw_functions
from .enums import *
from .exception import check_result
from .typedefs import *


def get_instance_proc_addr(
    instance: Instance,
    name: str,
) -> PFN_xrVoidFunction:
    """"""
    if name is not None:
        name = name.encode()
    function = PFN_xrVoidFunction()
    fxn = raw_functions.xrGetInstanceProcAddr
    result = check_result(fxn(
        instance,
        name,
        byref(function),
    ))
    if result.is_exception():
        raise result
    return function


def enumerate_api_layer_properties(
) -> Array:
    """"""
    property_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateApiLayerProperties
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        0,
        byref(property_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    properties = (ApiLayerProperties * property_capacity_input.value)(*([ApiLayerProperties()] * property_capacity_input.value))
    result = check_result(fxn(
        property_capacity_input,
        byref(property_capacity_input),
        properties,
    ))
    if result.is_exception():
        raise result
    return properties


def enumerate_instance_extension_properties(
    layer_name: str = None,
) -> Array:
    """"""
    if layer_name is not None:
        layer_name = layer_name.encode()
    property_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateInstanceExtensionProperties
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        layer_name,
        0,
        byref(property_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    properties = (ExtensionProperties * property_capacity_input.value)(*([ExtensionProperties()] * property_capacity_input.value))
    result = check_result(fxn(
        layer_name,
        property_capacity_input,
        byref(property_capacity_input),
        properties,
    ))
    if result.is_exception():
        raise result
    return properties


def create_instance(
    create_info: InstanceCreateInfo = None,
) -> Instance:
    """"""
    if create_info is None:
        create_info = InstanceCreateInfo()
    instance = Instance()
    fxn = raw_functions.xrCreateInstance
    result = check_result(fxn(
        create_info,
        byref(instance),
    ))
    if result.is_exception():
        raise result
    return instance


def destroy_instance(
    instance: Instance,
) -> None:
    """"""
    fxn = raw_functions.xrDestroyInstance
    result = check_result(fxn(
        instance,
    ))
    if result.is_exception():
        raise result


def get_instance_properties(
    instance: Instance,
) -> InstanceProperties:
    """"""
    instance_properties = InstanceProperties()
    fxn = raw_functions.xrGetInstanceProperties
    result = check_result(fxn(
        instance,
        byref(instance_properties),
    ))
    if result.is_exception():
        raise result
    return instance_properties


def poll_event(
    instance: Instance,
) -> EventDataBuffer:
    """"""
    event_data = EventDataBuffer()
    fxn = raw_functions.xrPollEvent
    result = check_result(fxn(
        instance,
        byref(event_data),
    ))
    if result.is_exception():
        raise result
    return event_data


def result_to_string(
    instance: Instance,
    value: Result,
    buffer: (c_char * 64),
) -> None:
    """"""
    fxn = raw_functions.xrResultToString
    result = check_result(fxn(
        instance,
        value.value,
        buffer,
    ))
    if result.is_exception():
        raise result


def structure_type_to_string(
    instance: Instance,
    value: StructureType,
    buffer: (c_char * 64),
) -> None:
    """"""
    fxn = raw_functions.xrStructureTypeToString
    result = check_result(fxn(
        instance,
        value.value,
        buffer,
    ))
    if result.is_exception():
        raise result


def get_system(
    instance: Instance,
    get_info: SystemGetInfo,
) -> SystemId:
    """"""
    system_id = SystemId()
    fxn = raw_functions.xrGetSystem
    result = check_result(fxn(
        instance,
        get_info,
        byref(system_id),
    ))
    if result.is_exception():
        raise result
    return system_id


def get_system_properties(
    instance: Instance,
    system_id: SystemId,
) -> SystemProperties:
    """"""
    properties = SystemProperties()
    fxn = raw_functions.xrGetSystemProperties
    result = check_result(fxn(
        instance,
        system_id,
        byref(properties),
    ))
    if result.is_exception():
        raise result
    return properties


def enumerate_environment_blend_modes(
    instance: Instance,
    system_id: SystemId,
    view_configuration_type: ViewConfigurationType,
) -> Array:
    """"""
    environment_blend_mode_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateEnvironmentBlendModes
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type.value,
        0,
        byref(environment_blend_mode_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    environment_blend_modes = (EnvironmentBlendMode.ctype() * environment_blend_mode_capacity_input.value)(*([EnvironmentBlendMode.ctype()()] * environment_blend_mode_capacity_input.value))
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type.value,
        environment_blend_mode_capacity_input,
        byref(environment_blend_mode_capacity_input),
        environment_blend_modes,
    ))
    if result.is_exception():
        raise result
    return environment_blend_modes


def create_session(
    instance: Instance,
    create_info: SessionCreateInfo = None,
) -> Session:
    """"""
    if create_info is None:
        create_info = SessionCreateInfo()
    session = Session()
    fxn = raw_functions.xrCreateSession
    result = check_result(fxn(
        instance,
        create_info,
        byref(session),
    ))
    if result.is_exception():
        raise result
    return session


def destroy_session(
    session: Session,
) -> None:
    """"""
    fxn = raw_functions.xrDestroySession
    result = check_result(fxn(
        session,
    ))
    if result.is_exception():
        raise result


def enumerate_reference_spaces(
    session: Session,
) -> Array:
    """"""
    space_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateReferenceSpaces
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        session,
        0,
        byref(space_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    spaces = (ReferenceSpaceType.ctype() * space_capacity_input.value)(*([ReferenceSpaceType.ctype()()] * space_capacity_input.value))
    result = check_result(fxn(
        session,
        space_capacity_input,
        byref(space_capacity_input),
        spaces,
    ))
    if result.is_exception():
        raise result
    return spaces


def create_reference_space(
    session: Session,
    create_info: ReferenceSpaceCreateInfo = None,
) -> Space:
    """"""
    if create_info is None:
        create_info = ReferenceSpaceCreateInfo()
    space = Space()
    fxn = raw_functions.xrCreateReferenceSpace
    result = check_result(fxn(
        session,
        create_info,
        byref(space),
    ))
    if result.is_exception():
        raise result
    return space


def get_reference_space_bounds_rect(
    session: Session,
    reference_space_type: ReferenceSpaceType,
) -> Extent2Df:
    """"""
    bounds = Extent2Df()
    fxn = raw_functions.xrGetReferenceSpaceBoundsRect
    result = check_result(fxn(
        session,
        reference_space_type.value,
        byref(bounds),
    ))
    if result.is_exception():
        raise result
    return bounds


def create_action_space(
    session: Session,
    create_info: ActionSpaceCreateInfo = None,
) -> Space:
    """"""
    if create_info is None:
        create_info = ActionSpaceCreateInfo()
    space = Space()
    fxn = raw_functions.xrCreateActionSpace
    result = check_result(fxn(
        session,
        create_info,
        byref(space),
    ))
    if result.is_exception():
        raise result
    return space


def locate_space(
    space: Space,
    base_space: Space,
    time: Time,
) -> SpaceLocation:
    """"""
    location = SpaceLocation()
    fxn = raw_functions.xrLocateSpace
    result = check_result(fxn(
        space,
        base_space,
        time,
        byref(location),
    ))
    if result.is_exception():
        raise result
    return location


def destroy_space(
    space: Space,
) -> None:
    """"""
    fxn = raw_functions.xrDestroySpace
    result = check_result(fxn(
        space,
    ))
    if result.is_exception():
        raise result


def enumerate_view_configurations(
    instance: Instance,
    system_id: SystemId,
) -> Array:
    """"""
    view_configuration_type_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateViewConfigurations
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        instance,
        system_id,
        0,
        byref(view_configuration_type_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    view_configuration_types = (ViewConfigurationType.ctype() * view_configuration_type_capacity_input.value)(*([ViewConfigurationType.ctype()()] * view_configuration_type_capacity_input.value))
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type_capacity_input,
        byref(view_configuration_type_capacity_input),
        view_configuration_types,
    ))
    if result.is_exception():
        raise result
    return view_configuration_types


def get_view_configuration_properties(
    instance: Instance,
    system_id: SystemId,
    view_configuration_type: ViewConfigurationType,
) -> ViewConfigurationProperties:
    """"""
    configuration_properties = ViewConfigurationProperties()
    fxn = raw_functions.xrGetViewConfigurationProperties
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type.value,
        byref(configuration_properties),
    ))
    if result.is_exception():
        raise result
    return configuration_properties


def enumerate_view_configuration_views(
    instance: Instance,
    system_id: SystemId,
    view_configuration_type: ViewConfigurationType,
) -> Array:
    """"""
    view_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateViewConfigurationViews
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type.value,
        0,
        byref(view_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    views = (ViewConfigurationView * view_capacity_input.value)(*([ViewConfigurationView()] * view_capacity_input.value))
    result = check_result(fxn(
        instance,
        system_id,
        view_configuration_type.value,
        view_capacity_input,
        byref(view_capacity_input),
        views,
    ))
    if result.is_exception():
        raise result
    return views


def enumerate_swapchain_formats(
    session: Session,
) -> Array:
    """"""
    format_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateSwapchainFormats
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        session,
        0,
        byref(format_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    formats = (c_int64 * format_capacity_input.value)(*([c_int64()] * format_capacity_input.value))
    result = check_result(fxn(
        session,
        format_capacity_input,
        byref(format_capacity_input),
        formats,
    ))
    if result.is_exception():
        raise result
    return formats


def create_swapchain(
    session: Session,
    create_info: SwapchainCreateInfo = None,
) -> Swapchain:
    """"""
    if create_info is None:
        create_info = SwapchainCreateInfo()
    swapchain = Swapchain()
    fxn = raw_functions.xrCreateSwapchain
    result = check_result(fxn(
        session,
        create_info,
        byref(swapchain),
    ))
    if result.is_exception():
        raise result
    return swapchain


def destroy_swapchain(
    swapchain: Swapchain,
) -> None:
    """"""
    fxn = raw_functions.xrDestroySwapchain
    result = check_result(fxn(
        swapchain,
    ))
    if result.is_exception():
        raise result


def enumerate_swapchain_images(
    swapchain: Swapchain,
    element_type: type,
) -> Array:
    """"""
    image_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateSwapchainImages
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        swapchain,
        0,
        byref(image_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    images = (element_type * image_capacity_input.value)(*([element_type()] * image_capacity_input.value))
    result = check_result(fxn(
        swapchain,
        image_capacity_input,
        byref(image_capacity_input),
        cast(images, POINTER(SwapchainImageBaseHeader)),
    ))
    if result.is_exception():
        raise result
    return images


def acquire_swapchain_image(
    swapchain: Swapchain,
    acquire_info: SwapchainImageAcquireInfo = None,
) -> int:
    """"""
    index = c_uint32()
    fxn = raw_functions.xrAcquireSwapchainImage
    result = check_result(fxn(
        swapchain,
        acquire_info,
        byref(index),
    ))
    if result.is_exception():
        raise result
    return index.value


def wait_swapchain_image(
    swapchain: Swapchain,
    wait_info: SwapchainImageWaitInfo,
) -> None:
    """"""
    fxn = raw_functions.xrWaitSwapchainImage
    result = check_result(fxn(
        swapchain,
        wait_info,
    ))
    if result.is_exception():
        raise result


def release_swapchain_image(
    swapchain: Swapchain,
    release_info: SwapchainImageReleaseInfo = None,
) -> None:
    """"""
    fxn = raw_functions.xrReleaseSwapchainImage
    result = check_result(fxn(
        swapchain,
        release_info,
    ))
    if result.is_exception():
        raise result


def begin_session(
    session: Session,
    begin_info: SessionBeginInfo,
) -> None:
    """"""
    fxn = raw_functions.xrBeginSession
    result = check_result(fxn(
        session,
        begin_info,
    ))
    if result.is_exception():
        raise result


def end_session(
    session: Session,
) -> None:
    """"""
    fxn = raw_functions.xrEndSession
    result = check_result(fxn(
        session,
    ))
    if result.is_exception():
        raise result


def request_exit_session(
    session: Session,
) -> None:
    """"""
    fxn = raw_functions.xrRequestExitSession
    result = check_result(fxn(
        session,
    ))
    if result.is_exception():
        raise result


def wait_frame(
    session: Session,
    frame_wait_info: FrameWaitInfo = None,
) -> FrameState:
    """"""
    frame_state = FrameState()
    fxn = raw_functions.xrWaitFrame
    result = check_result(fxn(
        session,
        frame_wait_info,
        byref(frame_state),
    ))
    if result.is_exception():
        raise result
    return frame_state


def begin_frame(
    session: Session,
    frame_begin_info: FrameBeginInfo = None,
) -> None:
    """"""
    fxn = raw_functions.xrBeginFrame
    result = check_result(fxn(
        session,
        frame_begin_info,
    ))
    if result.is_exception():
        raise result


def end_frame(
    session: Session,
    frame_end_info: FrameEndInfo,
) -> None:
    """"""
    fxn = raw_functions.xrEndFrame
    result = check_result(fxn(
        session,
        frame_end_info,
    ))
    if result.is_exception():
        raise result


def locate_views(
    session: Session,
    view_locate_info: ViewLocateInfo,
) -> (ViewState, Array):
    """"""
    view_state = ViewState()
    view_capacity_input = c_uint32(0)
    fxn = raw_functions.xrLocateViews
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        session,
        view_locate_info,
        byref(view_state),
        0,
        byref(view_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    views = (View * view_capacity_input.value)(*([View()] * view_capacity_input.value))
    result = check_result(fxn(
        session,
        view_locate_info,
        byref(view_state),
        view_capacity_input,
        byref(view_capacity_input),
        views,
    ))
    if result.is_exception():
        raise result
    return view_state, views


def string_to_path(
    instance: Instance,
    path_string: str,
) -> Path:
    """"""
    if path_string is not None:
        path_string = path_string.encode()
    path = Path()
    fxn = raw_functions.xrStringToPath
    result = check_result(fxn(
        instance,
        path_string,
        byref(path),
    ))
    if result.is_exception():
        raise result
    return path


def path_to_string(
    instance: Instance,
    path: Path,
) -> str:
    """"""
    buffer_capacity_input = c_uint32(0)
    fxn = raw_functions.xrPathToString
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        instance,
        path,
        0,
        byref(buffer_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    buffer = create_string_buffer(buffer_capacity_input.value)
    result = check_result(fxn(
        instance,
        path,
        buffer_capacity_input,
        byref(buffer_capacity_input),
        buffer,
    ))
    if result.is_exception():
        raise result
    return buffer.value.decode()


def create_action_set(
    instance: Instance,
    create_info: ActionSetCreateInfo = None,
) -> ActionSet:
    """"""
    if create_info is None:
        create_info = ActionSetCreateInfo()
    action_set = ActionSet()
    fxn = raw_functions.xrCreateActionSet
    result = check_result(fxn(
        instance,
        create_info,
        byref(action_set),
    ))
    if result.is_exception():
        raise result
    return action_set


def destroy_action_set(
    action_set: ActionSet,
) -> None:
    """"""
    fxn = raw_functions.xrDestroyActionSet
    result = check_result(fxn(
        action_set,
    ))
    if result.is_exception():
        raise result


def create_action(
    action_set: ActionSet,
    create_info: ActionCreateInfo = None,
) -> Action:
    """"""
    if create_info is None:
        create_info = ActionCreateInfo()
    action = Action()
    fxn = raw_functions.xrCreateAction
    result = check_result(fxn(
        action_set,
        create_info,
        byref(action),
    ))
    if result.is_exception():
        raise result
    return action


def destroy_action(
    action: Action,
) -> None:
    """"""
    fxn = raw_functions.xrDestroyAction
    result = check_result(fxn(
        action,
    ))
    if result.is_exception():
        raise result


def suggest_interaction_profile_bindings(
    instance: Instance,
    suggested_bindings: InteractionProfileSuggestedBinding,
) -> None:
    """"""
    fxn = raw_functions.xrSuggestInteractionProfileBindings
    result = check_result(fxn(
        instance,
        suggested_bindings,
    ))
    if result.is_exception():
        raise result


def attach_session_action_sets(
    session: Session,
    attach_info: SessionActionSetsAttachInfo,
) -> None:
    """"""
    fxn = raw_functions.xrAttachSessionActionSets
    result = check_result(fxn(
        session,
        attach_info,
    ))
    if result.is_exception():
        raise result


def get_current_interaction_profile(
    session: Session,
    top_level_user_path: Path,
) -> InteractionProfileState:
    """"""
    interaction_profile = InteractionProfileState()
    fxn = raw_functions.xrGetCurrentInteractionProfile
    result = check_result(fxn(
        session,
        top_level_user_path,
        byref(interaction_profile),
    ))
    if result.is_exception():
        raise result
    return interaction_profile


def get_action_state_boolean(
    session: Session,
    get_info: ActionStateGetInfo,
) -> ActionStateBoolean:
    """"""
    state = ActionStateBoolean()
    fxn = raw_functions.xrGetActionStateBoolean
    result = check_result(fxn(
        session,
        get_info,
        byref(state),
    ))
    if result.is_exception():
        raise result
    return state


def get_action_state_float(
    session: Session,
    get_info: ActionStateGetInfo,
) -> ActionStateFloat:
    """"""
    state = ActionStateFloat()
    fxn = raw_functions.xrGetActionStateFloat
    result = check_result(fxn(
        session,
        get_info,
        byref(state),
    ))
    if result.is_exception():
        raise result
    return state


def get_action_state_vector2f(
    session: Session,
    get_info: ActionStateGetInfo,
) -> ActionStateVector2f:
    """"""
    state = ActionStateVector2f()
    fxn = raw_functions.xrGetActionStateVector2f
    result = check_result(fxn(
        session,
        get_info,
        byref(state),
    ))
    if result.is_exception():
        raise result
    return state


def get_action_state_pose(
    session: Session,
    get_info: ActionStateGetInfo,
) -> ActionStatePose:
    """"""
    state = ActionStatePose()
    fxn = raw_functions.xrGetActionStatePose
    result = check_result(fxn(
        session,
        get_info,
        byref(state),
    ))
    if result.is_exception():
        raise result
    return state


def sync_actions(
    session: Session,
    sync_info: ActionsSyncInfo,
) -> None:
    """"""
    fxn = raw_functions.xrSyncActions
    result = check_result(fxn(
        session,
        sync_info,
    ))
    if result.is_exception():
        raise result


def enumerate_bound_sources_for_action(
    session: Session,
    enumerate_info: BoundSourcesForActionEnumerateInfo,
) -> Array:
    """"""
    source_capacity_input = c_uint32(0)
    fxn = raw_functions.xrEnumerateBoundSourcesForAction
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        session,
        enumerate_info,
        0,
        byref(source_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    sources = (Path * source_capacity_input.value)(*([Path()] * source_capacity_input.value))
    result = check_result(fxn(
        session,
        enumerate_info,
        source_capacity_input,
        byref(source_capacity_input),
        sources,
    ))
    if result.is_exception():
        raise result
    return sources


def get_input_source_localized_name(
    session: Session,
    get_info: InputSourceLocalizedNameGetInfo,
) -> str:
    """"""
    buffer_capacity_input = c_uint32(0)
    fxn = raw_functions.xrGetInputSourceLocalizedName
    # First call of two, to retrieve buffer sizes
    result = check_result(fxn(
        session,
        get_info,
        0,
        byref(buffer_capacity_input),
        None,
    ))
    if result.is_exception():
        raise result
    buffer = create_string_buffer(buffer_capacity_input.value)
    result = check_result(fxn(
        session,
        get_info,
        buffer_capacity_input,
        byref(buffer_capacity_input),
        buffer,
    ))
    if result.is_exception():
        raise result
    return buffer.value.decode()


def apply_haptic_feedback(
    session: Session,
    haptic_action_info: HapticActionInfo,
    haptic_feedback: HapticBaseHeader,
) -> None:
    """"""
    fxn = raw_functions.xrApplyHapticFeedback
    result = check_result(fxn(
        session,
        haptic_action_info,
        haptic_feedback,
    ))
    if result.is_exception():
        raise result


def stop_haptic_feedback(
    session: Session,
    haptic_action_info: HapticActionInfo,
) -> None:
    """"""
    fxn = raw_functions.xrStopHapticFeedback
    result = check_result(fxn(
        session,
        haptic_action_info,
    ))
    if result.is_exception():
        raise result


__all__ = [
    "acquire_swapchain_image",
    "apply_haptic_feedback",
    "attach_session_action_sets",
    "begin_frame",
    "begin_session",
    "create_action",
    "create_action_set",
    "create_action_space",
    "create_instance",
    "create_reference_space",
    "create_session",
    "create_swapchain",
    "destroy_action",
    "destroy_action_set",
    "destroy_instance",
    "destroy_session",
    "destroy_space",
    "destroy_swapchain",
    "end_frame",
    "end_session",
    "enumerate_api_layer_properties",
    "enumerate_bound_sources_for_action",
    "enumerate_environment_blend_modes",
    "enumerate_instance_extension_properties",
    "enumerate_reference_spaces",
    "enumerate_swapchain_formats",
    "enumerate_swapchain_images",
    "enumerate_view_configuration_views",
    "enumerate_view_configurations",
    "get_action_state_boolean",
    "get_action_state_float",
    "get_action_state_pose",
    "get_action_state_vector2f",
    "get_current_interaction_profile",
    "get_input_source_localized_name",
    "get_instance_proc_addr",
    "get_instance_properties",
    "get_reference_space_bounds_rect",
    "get_system",
    "get_system_properties",
    "get_view_configuration_properties",
    "locate_space",
    "locate_views",
    "path_to_string",
    "poll_event",
    "release_swapchain_image",
    "request_exit_session",
    "result_to_string",
    "stop_haptic_feedback",
    "string_to_path",
    "structure_type_to_string",
    "suggest_interaction_profile_bindings",
    "sync_actions",
    "wait_frame",
    "wait_swapchain_image",
]
